"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Context_1 = __importDefault(require("./Context"));
class MotionEffect {
    constructor(config) {
        this._config = config;
        this._measureContext = new Context_1.default(this._config);
    }
    get motion() {
        return this._motion;
    }
    get motionFunction() {
        return this._motionFunction;
    }
    setMotion(motion) {
        this._motion = motion;
        this._motionFunction = this.getMotionFunction();
    }
    getMotionFunction() {
        switch (this.motion) {
            case "none":
                return this.renderNone;
            case "scroll":
                return this.renderScroll;
            case "shake":
                return (input) => this.renderWave(input, this.getShake());
            case "slide":
                return this._config.version === "osrs"
                    ? (input) => this.renderSlide(input, this.getSlideOsrs())
                    : (input) => this.renderSlide(input, this.getSlideRs3());
            case "wave":
                return (input) => this.renderWave(input, this.getWave());
            case "wave2":
                return (input) => this.renderWave(input, this.getWave2());
        }
    }
    renderNone({ fragment, lineCharacterIndex }) {
        const { width: x } = this._measureContext.measureText(fragment.slice(0, lineCharacterIndex));
        const { ascent: y } = this._measureContext.measureText(fragment);
        return { x, y };
    }
    renderScroll({ fragment, frame, lineCharacterIndex, }) {
        const { ascent: y, width: lineWidth } = this._measureContext.measureText(fragment);
        const { width } = this._measureContext.measureText(fragment.slice(0, lineCharacterIndex));
        const displacement = Math.round((1 - (2 * frame) / this._config.totalFrames) * lineWidth);
        const x = width + displacement;
        return { x, y };
    }
    renderSlide({ fragment, frame, lineCharacterIndex }, { getY }) {
        const { width: x } = this._measureContext.measureText(fragment.slice(0, lineCharacterIndex));
        const { ascent, height } = this._measureContext.measureText(fragment);
        const motionFrameIndex = Math.round(this._config.totalFrames / 6);
        const y = getY({ ascent, frame, height, motionFrameIndex });
        return { x, y };
    }
    renderWave({ fragment, frame, lineCharacterIndex }, { amplitudeFactor, frameFactor, getWave, getX }) {
        const { ascent, height } = this._measureContext.measureText(fragment);
        const { width } = this._measureContext.measureText(fragment.slice(0, lineCharacterIndex));
        const amplitude = height * amplitudeFactor;
        const wave = Math.sin(Math.PI *
            (lineCharacterIndex / 6 +
                8 * frameFactor * (frame / this._config.totalFrames)));
        const displacement = amplitude * getWave({ frame, wave });
        const x = getX({ displacement, width });
        const y = Math.round(ascent + displacement);
        return { x, y };
    }
    getSlideOsrs() {
        return {
            getY: ({ ascent, frame, height, motionFrameIndex }) => {
                let displacement = ascent;
                if (frame < motionFrameIndex) {
                    displacement -=
                        ((motionFrameIndex - frame) / motionFrameIndex) * height;
                }
                else if (frame > this._config.totalFrames - motionFrameIndex) {
                    displacement -=
                        ((this._config.totalFrames - motionFrameIndex - frame) /
                            motionFrameIndex) *
                            height;
                }
                return Math.round(displacement);
            },
        };
    }
    getSlideRs3() {
        return {
            getY: ({ ascent, frame, height, motionFrameIndex }) => {
                let displacement = ascent;
                if (frame < motionFrameIndex) {
                    displacement +=
                        ((motionFrameIndex - frame) / motionFrameIndex) * height;
                }
                else if (frame > this._config.totalFrames - motionFrameIndex) {
                    displacement +=
                        ((this._config.totalFrames - motionFrameIndex - frame) /
                            motionFrameIndex) *
                            height;
                }
                return Math.round(displacement);
            },
        };
    }
    getShake() {
        return {
            amplitudeFactor: 1 / 3,
            frameFactor: 6,
            getWave: ({ frame, wave }) => 2 * frame > this._config.totalFrames
                ? 1
                : 1 + wave * (1 - (2 * frame) / this._config.totalFrames),
            getX: ({ width }) => width,
        };
    }
    getWave() {
        return {
            amplitudeFactor: 1 / 3,
            frameFactor: 1,
            getWave: ({ wave }) => 1 + wave,
            getX: ({ width }) => width,
        };
    }
    getWave2() {
        return {
            amplitudeFactor: 1 / 6,
            frameFactor: 1,
            getWave: ({ wave }) => 1 + wave,
            getX: ({ displacement, width }) => Math.round(width + displacement),
        };
    }
}
exports.default = MotionEffect;
