"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const word_wrap_1 = __importDefault(require("word-wrap"));
const EffectsUtil_1 = require("../utils/EffectsUtil");
const UNSUPPORTED_FONT_CHARACTERS_REGEXP = /([^ -~\t\n]|`)+/g;
class Parser {
    constructor(config, wordWrapConfig) {
        this._config = config;
        this._wordWrapConfig = wordWrapConfig;
    }
    parse(string) {
        const sanitizedMessage = this.sanitizeMessage(string);
        const effectsString = this.getEffectsString(string);
        const message = this.formatMessage(sanitizedMessage, effectsString);
        const effects = this.getMessageEffects(effectsString);
        return Object.assign(Object.assign({}, effects), { message });
    }
    sanitizeMessage(string) {
        if (string === "") {
            throw Error("The message string cannot be empty.");
        }
        if (string.trim() === "") {
            throw Error("The message string cannot consist of only whitespaces.");
        }
        return string
            .replace(UNSUPPORTED_FONT_CHARACTERS_REGEXP, this._config.replacement)
            .slice(0, this._config.maxMessageLength);
    }
    getEffectsString(string) {
        var _a;
        const escapedSuffix = "\\" + this._config.suffix.split("").join("\\");
        const patternString = `pattern[a-z0-9]{1,8}`;
        const colorString = `(${patternString}|${EffectsUtil_1.COLORS.join("|")})` + escapedSuffix;
        const motionString = `(${EffectsUtil_1.MOTIONS.join("|")})` + escapedSuffix;
        const effectsRegExp = RegExp(`^(${colorString}(${motionString})?|${motionString})`, "i");
        return ((_a = string.match(effectsRegExp)) !== null && _a !== void 0 ? _a : [""])[0];
    }
    formatMessage(string, effectsString) {
        string = string.replace(effectsString, "");
        if (string === "") {
            throw new Error("The effects cannot be applied to a message string that is empty.");
        }
        if (string.trim() === "") {
            throw new Error("The effects cannot be applied to a message string that consists of only whitespaces.");
        }
        return (0, word_wrap_1.default)(this._config.enforceCapitalization
            ? this.applyCapitalization(string)
            : string, this._wordWrapConfig);
    }
    getMessageEffects(effectsString) {
        return effectsString
            .toLowerCase()
            .split(this._config.suffix)
            .filter(Boolean)
            .reduce((value, effect) => {
            if (effect.startsWith("pattern")) {
                return Object.assign(Object.assign({}, value), { color: "pattern", pattern: effect.replace("pattern", "").split("") });
            }
            return Object.assign(Object.assign({}, value), { [EffectsUtil_1.EFFECTS_MAP[effect]]: effect });
        }, {
            color: this._config.color,
            motion: this._config.motion,
            pattern: this._config.pattern,
        });
    }
    applyCapitalization(string) {
        let capitalizeNextWord = true;
        return string
            .split(" ")
            .map((word) => {
            if (word === "") {
                return "";
            }
            const returnWord = (capitalizeNextWord ? word.charAt(0).toUpperCase() : word.charAt(0)) +
                word.slice(1).toLowerCase();
            capitalizeNextWord = [".", "!", "?"].some((punctuation) => word.endsWith(punctuation));
            return returnWord;
        })
            .join(" ");
    }
}
exports.default = Parser;
