"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const EffectsUtil_1 = require("../utils/EffectsUtil");
const range_1 = __importDefault(require("../utils/range"));
const Context_1 = __importDefault(require("./Context"));
class Renderer {
    constructor(config) {
        this._config = config;
        this._measureContext = new Context_1.default(this._config);
    }
    setEffects(colorEffect, motionEffect) {
        this._colorEffect = colorEffect;
        this._motionEffect = motionEffect;
    }
    render(message) {
        const contexts = [];
        const framesCount = (0, EffectsUtil_1.isAnimated)(this._colorEffect.color, this._motionEffect.motion)
            ? this._config.totalFrames
            : 1;
        const processPerLine = (0, EffectsUtil_1.requiresProcessingPerLine)(this._colorEffect.color, this._motionEffect.motion);
        const processPerCharacter = (0, EffectsUtil_1.requiresProcessingPerCharacter)(this._colorEffect.color, this._motionEffect.motion);
        for (const frame of (0, range_1.default)(framesCount)) {
            if (!processPerLine) {
                const messageContext = this.getContext(message);
                this.renderFrame({ context: messageContext, fragment: message, frame });
                contexts.push(messageContext.context);
                continue;
            }
            let messageCharacterIndex = 0;
            let messageContext = new Context_1.default(this._config);
            for (const line of message.split("\n")) {
                const lineContext = this.getContext(line);
                if (!processPerCharacter) {
                    this.renderFrame({ context: lineContext, fragment: line, frame });
                    messageContext = this.mergeContexts(messageContext, lineContext);
                    continue;
                }
                let lineCharacterIndex = 0;
                for (const character of line.split("")) {
                    this.renderFrame({
                        character,
                        context: lineContext,
                        fragment: line,
                        frame,
                        index: messageCharacterIndex,
                        lineCharacterIndex,
                    });
                    ++messageCharacterIndex;
                    ++lineCharacterIndex;
                }
                messageContext = this.mergeContexts(messageContext, lineContext);
            }
            contexts.push(messageContext.context);
        }
        return contexts;
    }
    renderFrame({ character, context, fragment, frame, index = 0, lineCharacterIndex = 0, }) {
        const { x, y } = this._motionEffect.motionFunction({
            fragment,
            frame,
            lineCharacterIndex,
        });
        context.fill({
            colorEffect: this._colorEffect,
            fragment: character !== null && character !== void 0 ? character : fragment,
            frame,
            index,
            x,
            y,
        });
    }
    getContext(line) {
        const { height, width } = this.getDimensions(line);
        return new Context_1.default(this._config, width, height).initialize();
    }
    getDimensions(line) {
        const { height, width } = this._measureContext.measureText(line);
        if (["none", "scroll", "slide"].includes(this._motionEffect.motion)) {
            return { height, width };
        }
        if (["shake", "wave"].includes(this._motionEffect.motion)) {
            const amplitude = height / 3;
            return { height: Math.round(height + 2 * amplitude), width };
        }
        const amplitude = height / 6;
        return {
            height: Math.round(height + 2 * amplitude),
            width: Math.round(width + 2 * amplitude),
        };
    }
    mergeContexts(messageContext, lineContext) {
        if (messageContext.context.canvas.height === 0) {
            return lineContext;
        }
        const maxWidth = Math.max(messageContext.context.canvas.width, lineContext.context.canvas.width);
        const totalHeight = messageContext.context.canvas.height + lineContext.context.canvas.height;
        const newContext = new Context_1.default(this._config, maxWidth, totalHeight);
        newContext.context.drawImage(messageContext.context.canvas, 0, 0);
        newContext.context.drawImage(lineContext.context.canvas, 0, messageContext.context.canvas.height);
        return newContext;
    }
}
exports.default = Renderer;
